(* ------------------------------------------------------------ *)
(* ---                   CDOSCALL.MOD                       --- *)
(* ---   Interface to System Interprocess message queues    --- *)
(* ---   and other functions. This version is for Clarion   --- *)
(* ---   for Windows V5.0a                                  --- *)
(* ---                                                      --- *)
(* ---   Version 1.03  Last modified by  CWS 08-05-00       --- *)
(* ---   Version 1.04  Last modified by  CWS 08-17-01       --- *)
(* --- Change 'CARDINAL(Var)' to the form 'VAL(CARDINAL, Var)'. *)
(* ------------------------------------------------------------ *)

(* Implemented for JPI Topspeed Clarion/Modula-2 V5.0 *)


IMPLEMENTATION MODULE CDOSCALL;

(*  # debug(vid => off) *)
(*# call(o_a_copy => off) *)

IMPORT Str, Clw, CWRUN;                       (* 01-13-00 *)

FROM CWRUN IMPORT SIGNED, UNSIGNED;           (* 04-06-99 *)

FROM CLWCALL IMPORT
  WindowManager, CLWCALL$WAITKEY, CLWCALL$TDELAY, CLWCALL$READDDE;   (* 08-05-99 *)




 (* ============================================= *)
 (*  Set the window pointer for this thread into *)
 (*  a storeage array for safekeeping. *)

 PROCEDURE SETTHREAD(Wnd:WindowManager): BOOLEAN;
 VAR Td: CARDINAL;
 BEGIN
   Td := VAL(CARDINAL, CWRUN.THREAD() );               (* 08-17-01 *)
   IF Td > MaxThread THEN RETURN FALSE; END;
   ThdW[Td] := Wnd;
   RETURN TRUE;
 END SETTHREAD;




 (* ============================================= *)
 (*  Wait for a Key Press to be received *)

 PROCEDURE WAITKEY();                                   (* 08-05-00 *)
 VAR Td: CARDINAL;  Wnd: WindowManager;
 BEGIN
   Td := VAL(CARDINAL, CWRUN.THREAD() );                (* 08-17-01 *)
   Wnd := ThdW[Td];
   CLWCALL$WAITKEY(Wnd);                                (* 08-05-00 *)
 END WAITKEY;




 (* ============================================= *)
 (*  delay for number of system ticks and allow *)
 (*  other processes to run. delay is plus or *)
 (*  minus one tick. *)

 PROCEDURE PDELAY(TICKS:CARDINAL);
 VAR Td: CARDINAL;  Wnd: WindowManager;
 BEGIN
   Td := VAL(CARDINAL, CWRUN.THREAD() );             (* 08-17-01 *)
   Wnd := ThdW[Td];
   CLWCALL$TDELAY(Wnd,TICKS);                        (* 04-08-99 *)
 END PDELAY;




 (* ============================================= *)
 (* Create System message queue as specified in arguments. *)
 (* Uses System Queue buffer area in sysdat memory page. *)
 (* Returns TRUE if successfull or FALSE if error. *)

 PROCEDURE MakeSysQue(QUENAME:ARRAY OF CHAR;
                        Qflags,QmLen,NMsgs:WORD; VAR QD:QDesc): CARDINAL;
 VAR vlen: CARDINAL;  Qptr: PointerToQDesc;
 BEGIN
   vlen := Str.Length(QUENAME);
   IF (vlen = 0) OR (vlen > 8) THEN RETURN MAX(CARDINAL); END;
   (* Initialize Queue Descriptor Block. *)
   (* Get QUEUE name and write to QD name field. *)
   WITH QD DO
     flags := Qflags;
     Str.Copy(que_name, QUENAME);
     WHILE vlen <= 7 DO que_name[vlen] := " "; INC(vlen); END;
     msg_len :=   QmLen;  (* word *)
     qbuffer := NIL;    (* use sysdat qbuf area *)
     next := NIL;
   END; (* with *)
   (* Link in new QD *)
   IF QDP = NIL THEN
     QDP := ADR(QD);        (* 03-30-99 *)
   ELSE
     Qptr := QDP;
     WHILE Qptr^.next # NIL DO Qptr := Qptr^.next; END;
     Qptr^.next := ADR(QD);     (* 03-30-99 *)
   END;
   (* Make DDE not needed in windows *)
   RETURN 0;                            (* 04-08-99 *)
 END MakeSysQue;



 (* ============================================= *)


 PROCEDURE OpenSysQue(QUENAME:ARRAY OF CHAR;
                       VAR QueBuffer:ARRAY OF BYTE; VAR QPB:QParBlk): CARDINAL;
 VAR vlen: CARDINAL;  Qptr: PointerToQDesc;  isServer: BOOLEAN;
 BEGIN
   vlen := Str.Length(QUENAME);
   IF (vlen = 0) OR (vlen > 8) THEN RETURN MAX(CARDINAL); END;  (* 04-27-92 *)
   (* Initialize Queue Parameter Block. *)
   (* Get QUEUE name and write to QPB name field. *)
   WITH QPB DO
    (* queue_id *) (* filled in below *)
     qmbuffer := ADR(QueBuffer);  (* qpb buffer pointer *)   (* 03-30-99 *)
     msg_len := HIGH(QueBuffer) + 1;                         (* 04-29-99 *)
     Str.Copy(qpb_name, QUENAME);                   (* 04-27-92 *)
     WHILE vlen <= 7 DO qpb_name[vlen] := " "; INC(vlen); END;  (* 04-30-92 *)
   END; (* with *)
   (* Look for QD to see if this is a server. *)
   isServer := FALSE;
   IF QDP # NIL THEN
     Qptr := QDP;
     WHILE Qptr # NIL DO
       (* string arrays are the same size *)
       IF QPB.qpb_name = Qptr^.que_name THEN
         isServer := TRUE;
       END;
       Qptr := Qptr^.next;
     END;
   END;
   (* Now, Open the queue *)
   IF isServer THEN
     QPB.queue_id := CWRUN.DDEserver(QUENAME,"");              (* 04-05-99 *)
     IF QPB.queue_id = 0 THEN RETURN NoQueFound; END;
   ELSE
     QPB.queue_id := CWRUN.DDEclient(QUENAME,"");                  (* 04-02-99 *)
     IF QPB.queue_id = 0 THEN RETURN NoQueFound; END;           (* 03-29-99 *)
   END;
   RETURN 0;
  END OpenSysQue;

 (* ============================================= *)

 (* Data is placed in buffer referenced by queue parameter block. *)
 PROCEDURE CReadSysQue(VAR QPB:QParBlk): CARDINAL;
 VAR Td: CARDINAL;  Wnd: WindowManager;  Retcd: CARDINAL;
 BEGIN
   Td := VAL(CARDINAL, CWRUN.THREAD() );                (* 08-17-01 *)
   Wnd := ThdW[Td];
   Retcd := CLWCALL$READDDE( Wnd, FALSE, QPB.queue_id, QPB.msg_len, QPB.qmbuffer^ );  (* 05-03-99 *)
   IF Retcd # 0 THEN RETURN Retcd;
   ELSE RETURN 0;
   END;
 END CReadSysQue;

 (* ============================================= *)


 (* Data is placed in buffer referenced by queue parameter block. *)
 PROCEDURE ReadSysQue(VAR QPB:QParBlk): CARDINAL;       (* 04-27-92 *)
 VAR Td: CARDINAL;  Wnd: WindowManager;  Retcd: CARDINAL;
 BEGIN
   Td := VAL(CARDINAL, CWRUN.THREAD() );                (* 08-17-01 *)
   Wnd := ThdW[Td];
   Retcd := CLWCALL$READDDE( Wnd, TRUE, QPB.queue_id, QPB.msg_len, QPB.qmbuffer^ );  (* 05-03-99 *)
   IF Retcd # 0 THEN RETURN Retcd;
   ELSE RETURN 0;
   END;
 END ReadSysQue;

 (* ============================================= *)

 (* Data is obtained from buffer referenced by queue parameter block. *)
 PROCEDURE CWriteSysQue(VAR QPB:QParBlk): CARDINAL;     (* 04-27-92 *)
 BEGIN
   Clw.Cla$PushString(QPB.qmbuffer^, QPB.msg_len);         (* 04-29-99 *)
   Clw.Cla$DDEexecute(QPB.queue_id);                       (* 04-29-99 *)
   RETURN 0;
 END CWriteSysQue;

 (* ============================================= *)


 (* Data is obtained from buffer referenced by queue parameter block. *)
 PROCEDURE WriteSysQue(VAR QPB:QParBlk): CARDINAL;      (* 04-27-92 *)
 BEGIN
   Clw.Cla$PushString(QPB.qmbuffer^, QPB.msg_len);         (* 04-29-99 *)
   Clw.Cla$DDEexecute(QPB.queue_id);                       (* 04-29-99 *)
   RETURN 0;
 END WriteSysQue;

 (* ============================================= *)

 PROCEDURE DelSysQue(VAR QPB:QParBlk): CARDINAL;        (* 04-27-92 *)
 BEGIN
   CWRUN.DDEclose(QPB.queue_id);            (* 04-02-99 *)
   RETURN 0;
 END DelSysQue;





 (* ============================================= *)
 (* Show text for CDOS system errors. *)

 PROCEDURE ShowError(Err:CARDINAL): ErStrType;
  VAR Ms: ErStrType;  ok: BOOLEAN;
  BEGIN
    IF Err = 0 THEN RETURN( "" ); END;  (* no error *)
    CASE Err OF
     |QueChanErr: RETURN("Error in system queue channel number.");
     |NoQueFound: RETURN("Cannot find system queue.");
     |QueInUse:   RETURN("System queue is in use.");         (* 05-05-99 *)
     |QueEmpty:   RETURN("Empty system queue.");
     |QueFull:    RETURN("Full system queue.");
     |QShutDown:  RETURN("Program shut down event.");        (* 05-05-99 *)
     |QLostEvent: RETURN("Program lost an unexpected event.");  (* 05-05-99 *)
    ELSE
      Str.CardToStr(VAL(LONGCARD,Err), Ms, 10, ok);
      Str.Prepend(Ms, "Sys Error ");
      Str.Append(Ms, " reported from system call.");
      RETURN( Ms );
    END; (* case *)
  END ShowError;




BEGIN
  QDP := NIL;

END CDOSCALL.
